// SPDX-License-Identifier: GPL-2.0
/*
 * linux/fs/ldiskfs/xattr_trusted.c
 * Handler for trusted extended attributes.
 *
 * Copyright (C) 2003 by Andreas Gruenbacher, <a.gruenbacher@computer.org>
 */

#include <linux/string.h>
#include <linux/capability.h>
#include <linux/fs.h>
#include "ldiskfs_jbd2.h"
#include "ldiskfs.h"
#include "xattr.h"

static bool
ldiskfs_xattr_trusted_list(struct dentry *dentry)
{
	return capable(CAP_SYS_ADMIN);
}

static int
ldiskfs_xattr_trusted_get(const struct xattr_handler *handler,
		       struct dentry *unused, struct inode *inode,
		       const char *name, void *buffer, size_t size)
{
	return ldiskfs_xattr_get(inode, LDISKFS_XATTR_INDEX_TRUSTED,
			      name, buffer, size);
}

static int
ldiskfs_xattr_trusted_set(const struct xattr_handler *handler,
		       struct dentry *unused, struct inode *inode,
		       const char *name, const void *value,
		       size_t size, int flags)
{
	return ldiskfs_xattr_set(inode, LDISKFS_XATTR_INDEX_TRUSTED,
			      name, value, size, flags);
}

const struct xattr_handler ldiskfs_xattr_trusted_handler = {
	.prefix	= XATTR_TRUSTED_PREFIX,
	.list	= ldiskfs_xattr_trusted_list,
	.get	= ldiskfs_xattr_trusted_get,
	.set	= ldiskfs_xattr_trusted_set,
};
